/*
  KatSplatter.c by Bertrand-Xavier M. March 2007
*/

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "HillTerrain/HillTerrain.h"
#include "splatter.h"

// Types =======================================================================

 // Heightmap, textures and Lightmap have in common a few properties:
 typedef struct
 {
   gchar *chemincomplet;	// chemin absolu sur le disque

   GtkWidget *frame;		// chaque IMG a son propre frame
   GtkWidget *tabl;		// et une table pour organiser les contrôles

   GdkPixbuf *pixbuf;		// image taille normale
   GdkPixbuf *minipixbuf;	// image miniature
   GtkWidget *btn;		// Bouton clic sur miniature
   GtkWidget *img;		// image qui contient minipixbuf, dans btn

   GtkWidget *labelnom;		// Label nom du fichier

   GtkWidget *labelTaille;	// Label taille
   GtkWidget *btnSuppr;		// "Remove" Button

   GtkWidget *labelAlt;		// Label Altitude
   GtkWidget *entry;		// altitude
 } IMG;
   IMG *IMGs[6];
		/*
		slot 0		: heightmap
  		slot 1,2,3,4	: textures
		slot 5		: lightmap
		*/

 // When config is loaded from disk, values are stored here. Then passed
 // To HillTerrain class constructor
 typedef struct
 {
   //unsigned short	Size;		//= 256;
   float		HillMin;	//= 2.0;
   float		HillMax;	//= 40.0;
   unsigned short	NumHills;	//= 200;
   unsigned short	Flatten;	//= 1;
   gboolean		Island;		//= FALSE;
   unsigned int		Seed;		//= 12345;
 } cfg;
   cfg *CFG;

// Global Variables ============================================================

 GtkWidget *fen, *tableHmap, *tableTex, *tableLmap;
 GtkWidget *mainIMG;	// image centrale

 guint nbTexChargees=0;	// Nombre de textures chargées actuellement 0-4
 gint currentID =-1;	// IMG actuellement affichée au centre. de 0 à 6. -1=aucun

 GtkWidget *entries[12];// Les deux Texture Factors (   0 et 1)
			// Les HillTerrain          (de 2 à  7)
			// Lightmap                 (de 8 à 11)

// Function Prototypes =========================================================

 void		Init_GUI		();
 GtkWidget	*CoolButton		(gchar *filename, gchar *caption);
 void		creer_labelEntry	(gchar *caption,
					 gint i, gchar* defval, gint maxlength,
					 GtkWidget *table, gint y);

 void		Load_Config		(char *configFile);

 void		Add_Texture		(GtkWidget *table);
 void		Add_Heightmap		(GtkWidget *table);
 void		Add_Lightmap		(GtkWidget *table);
 gchar*		Choisir_Fichier		(gchar *msg);
 void		Add_item		(GtkWidget *table, gchar *path, gint slot);

 void		clean_IMG		(gint i);
 void		Remove_IMG		(gpointer i);
 void		Miniature_Click		(gpointer i);

 void		Gen_HillTerrain		();
 static void	Gen_Texture		();
 void		Export_File		();

 void		App_Close		();

//==============================================================================



/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Entry Point
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int main (int argc, char *argv[])
{
   gtk_init (&argc, &argv);

   Init_GUI();
   Load_Config("config.txt");

   gtk_main();
   return(0);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Load Config
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Load_Config (char *configFile)
{
  FILE	 *pF;
  unsigned short n;
  guchar flag=0, b=0;
  gchar	 c, mot[200], *tmp=NULL;
  const char *mots[8]={"heightmap", "texture","fHillMin","fHillMax",
                       "uNumHills", "uFlattening","bIsland","uSeed"};

  CFG = (cfg*)g_malloc(sizeof(cfg));

  if (!(pF = fopen(configFile, "r"))) return;
  while ((c = fgetc (pF)) != EOF)
  {
   if (c == '=')
   {
     flag = 0;
     mot[b]=0;
     for (n=0; n<8; n++)
     {
       if (!strcmp(mot,mots[n])) { flag=n+1; break; }
     }
     memset(mot,0,b);  b=0;
   }
   else if (c ==0XA)
   {
     mot[b]=0;
     if      (flag==1)
     {
       tmp = g_locale_to_utf8 (mot, -1, NULL, NULL, NULL);
       Add_item(tableHmap, mot, ++currentID);
     }
     else if (flag==2)
     {
       tmp = g_locale_to_utf8 (mot, -1, NULL, NULL, NULL);
       Add_item(tableTex,  mot, ++currentID);
     }
     else if (flag==3)
     {
       tmp = g_locale_to_utf8 (mot, -1, NULL, NULL, NULL);
       gtk_entry_set_text (GTK_ENTRY (entries[2]), tmp);
       CFG->HillMin = atof(mot);
     }
     else if (flag==4)
     {
       tmp = g_locale_to_utf8 (mot, -1, NULL, NULL, NULL);
       gtk_entry_set_text (GTK_ENTRY (entries[3]), tmp);
       CFG->HillMax = atof(mot);
     }
     else if (flag==5)
     {
       tmp = g_locale_to_utf8 (mot, -1, NULL, NULL, NULL);
       gtk_entry_set_text (GTK_ENTRY (entries[4]), tmp);
       CFG->NumHills = atoi(mot);
     }
     else if (flag==6)
     {
       tmp = g_locale_to_utf8 (mot, -1, NULL, NULL, NULL);
       gtk_entry_set_text (GTK_ENTRY (entries[5]), tmp);
       CFG->Flatten = atoi(mot);
     }
     else if (flag==7)
     {
       tmp = g_locale_to_utf8 (mot, -1, NULL, NULL, NULL);
       gtk_entry_set_text (GTK_ENTRY (entries[6]), tmp);
       CFG->Island = atoi(mot);
     }
     else if (flag==8)
     {
       tmp = g_locale_to_utf8 (mot, -1, NULL, NULL, NULL);
       gtk_entry_set_text (GTK_ENTRY (entries[7]), tmp);
       CFG->Seed = atoi(mot);
     }
     memset(mot,0,b);  b=flag=0;
   }
   else if (c !=0XD) mot[b++] = c;
  }
  fclose (pF);
  if (tmp) g_free(tmp);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Gen Heightmap with HillTerrain CPP class
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Gen_HillTerrain()
{
  // Get size of final image
     gushort  Size=256;
     if      (IMGs[0]) Size = gdk_pixbuf_get_width(IMGs[0]->pixbuf);
     else if (IMGs[1]) Size = gdk_pixbuf_get_width(IMGs[1]->pixbuf);
     else if (IMGs[2]) Size = gdk_pixbuf_get_width(IMGs[2]->pixbuf);
     else if (IMGs[3]) Size = gdk_pixbuf_get_width(IMGs[3]->pixbuf);
     else if (IMGs[4]) Size = gdk_pixbuf_get_width(IMGs[4]->pixbuf);

  // Get values from fields 2 - 7
     CFG->HillMin  = atof(gtk_entry_get_text (GTK_ENTRY(entries[2])));
     CFG->HillMax  = atof(gtk_entry_get_text (GTK_ENTRY(entries[3])));
     CFG->NumHills = atoi(gtk_entry_get_text (GTK_ENTRY(entries[4])));
     CFG->Flatten  = atoi(gtk_entry_get_text (GTK_ENTRY(entries[5])));
     CFG->Island   = atoi(gtk_entry_get_text (GTK_ENTRY(entries[6])));
     CFG->Seed     = atoi(gtk_entry_get_text (GTK_ENTRY(entries[7])));

  // Start timer
     GTimer *Chrono = g_timer_new();

  // Here. Gen and export Hillterrain.bmp
     create_HillTerrain(Size, CFG->HillMin, CFG->HillMax, CFG->NumHills,
			CFG->Flatten, CFG->Island, CFG->Seed);

  // Calculate elapsed time
     g_timer_stop(Chrono);
     gchar *str;
     gdouble duree = g_timer_elapsed(Chrono, NULL);
     if      (duree < 60000)
       str = g_strdup_printf ("Heightmap Generated in %.3fs", duree);
     else
     {
       gdouble mn = duree/60000;
       gdouble s  = duree - (mn * 60000);
       str = g_strdup_printf ("Heightmap Generated in %.0fmn %.0fs", mn, s);
     }
     gtk_window_set_title (GTK_WINDOW (fen), str);
     g_timer_destroy (Chrono);
     g_free(str);

  // Then load it, but first clean already lopaded heightmap
     if (IMGs[0]) clean_IMG(0);
     Add_item(tableHmap, "HillTerrain.bmp", 0);

}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Splat !! Use one of these C++ classes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void Gen_Texture()
{
  // I must have 4 loaded textures
  //g_print("current id=%d nbTexChargees=%d\n", currentID, nbTexChargees);
  if (nbTexChargees < 4) return;
  char *img[4]={IMGs[1]->chemincomplet,
		IMGs[2]->chemincomplet,
		IMGs[3]->chemincomplet,
		IMGs[4]->chemincomplet};

  guchar alt[4];//={212,128,64,32};
         alt[0] = atoi(gtk_entry_get_text (GTK_ENTRY(IMGs[1]->entry)));
         alt[1] = atoi(gtk_entry_get_text (GTK_ENTRY(IMGs[2]->entry)));
         alt[2] = atoi(gtk_entry_get_text (GTK_ENTRY(IMGs[3]->entry)));
         alt[3] = atoi(gtk_entry_get_text (GTK_ENTRY(IMGs[4]->entry)));

  guchar AX = atoi(gtk_entry_get_text (GTK_ENTRY(entries[0])));
  guchar BX = atoi(gtk_entry_get_text (GTK_ENTRY(entries[1])));

  createSplatter(IMGs[0]->chemincomplet, img, alt, AX, BX);
  gtk_image_set_from_file (GTK_IMAGE(mainIMG), "final.bmp");
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Add Lightmap
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Add_Lightmap(GtkWidget *table)
{
/*
  Code by funto
  int ombrage = (int)gtk_adjustment_get_value(adj_ombrage);
  float angle_rayons = (int)gtk_adjustment_get_value(adj_angle_rayons);
  float hauteur_max_rayons = (int)gtk_adjustment_get_value(adj_hauteur_max_rayons);
  float hauteur_max_heightmap = (float)gtk_adjustment_get_value(adj_hauteur_max_heightmap);

  int n_channels;
  int rowstride_lightmap, rowstride_heightmap;
  guchar *pixels_lightmap=NULL, *p_lightmap=NULL;
  guchar *pixels_heightmap=NULL, *p_heightmap=NULL;
  int x, y, width, height = 0;
  float step_z=0.0f;
  float z_origine=0.0f;
  int x_rayon=0;
  float z_rayon=0.0f;

  n_channels = gdk_pixbuf_get_n_channels(Lightmap->pixbuf);

  g_assert(gdk_pixbuf_get_colorspace (Lightmap->pixbuf) == GDK_COLORSPACE_RGB);
  g_assert(gdk_pixbuf_get_bits_per_sample (Lightmap->pixbuf) == 8);
  g_assert(gdk_pixbuf_get_has_alpha (Lightmap->pixbuf) == FALSE);
  g_assert(n_channels == 3);

  width = gdk_pixbuf_get_width(Lightmap->pixbuf);
  height = gdk_pixbuf_get_height(Lightmap->pixbuf);

  rowstride_lightmap = gdk_pixbuf_get_rowstride(Lightmap->pixbuf);
  pixels_lightmap = gdk_pixbuf_get_pixels(Lightmap->pixbuf);

  rowstride_heightmap = gdk_pixbuf_get_rowstride(Heightmap->pixbuf);
  pixels_heightmap = gdk_pixbuf_get_pixels(Heightmap->pixbuf);

  // Au départ, tout est dans l'ombre...
  for(y=0 ; y<height ; y++)
    for(x=0 ; x<width ; x++)
    {
      p_lightmap = pixels_lightmap + y * rowstride_lightmap + x * n_channels;
      p_lightmap[0] = ombrage;
      p_lightmap[1] = ombrage;
      p_lightmap[2] = ombrage;
    }

  // Puis vint la lumière !!

  // calcule de combien on doit incrémenter en z à chaque nouveau rayon
  step_z = fabs(1/tan(angle_rayons));

  // lance les rayons dans le sens horizontal (en voyant de haut), de la
  //   gauche vers la droite. fait donc ça pour toutes les lignes (d'où la
  //   boucle en y). 
  for(y=0 ; y<height ; y++)
  {
    // parcourt en montant (i.e. suivant l'axe des Z) pour lancer des rayons
    // à chacune de ces hauteurs
    for(z_origine=0.0f ; z_origine<hauteur_max_rayons ; z_origine+=step_z)
    {
      // Lancer du rayon !
      for(z_rayon=z_origine, x_rayon=0 ;
			x_rayon < width;
			z_rayon-=step_z, x_rayon++)
      {
        // Si la hauteur du pixel en cours est supérieure ou égale à z_rayon,
        //   c'est qu'on a touché qqch, alors on le colorie en blanc et on passe
        //   au rayon d'après

        // On pointe sur le point (x_rayon, y)
        p_heightmap = pixels_heightmap + y * rowstride_heightmap + x_rayon * n_channels;
        p_lightmap = pixels_lightmap + y * rowstride_lightmap + x_rayon * n_channels;

        if((float)(p_heightmap[0]/255.0f)*hauteur_max_heightmap >= z_rayon)
        {
           p_lightmap[0] = p_lightmap[1] = p_lightmap[2] = 255;
           break;
        }
      }
    }
  }

  gtk_image_set_from_pixbuf(GTK_IMAGE(Lightmap->image), Lightmap->pixbuf);
  redimensionner_image(Lightmap->image);
*/
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Save big central image
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Export_File()
{
  // Ask user to type name and extension
  GtkWidget *dlg = gtk_file_chooser_dialog_new ("Enregistrer Sous (jpg ou png) ...",
			NULL, GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);

  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dlg), "tmp.png");

  if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_ACCEPT)
  {
    gchar *chem = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
    if (!chem) return;

    GdkPixbuf* tmp = gtk_image_get_pixbuf (GTK_IMAGE(mainIMG));
    if (!tmp) return;

    // extract extension that he typed
    gint n, len, pos;
    n = pos = strlen(chem);

    while(pos>0) // look for the position of the last dot
    {
     if (chem[pos]=='.') break;
     pos--;
    }

    if (pos) // if there's a dot,
    {
      len = n-pos; // get extension ...
      gchar *exta = g_malloc(len);
      strncpy(exta, chem+pos+1, len-1);
      exta[len-1]='\0';

      if      (!strcmp(exta, "bmp")); // si bmp, appeler la classe bitmap!
      else if (!strcmp(exta, "jpg"))
            gdk_pixbuf_save (tmp, chem, "jpeg", NULL,"quality","100", NULL);
      else  gdk_pixbuf_save (tmp, chem, exta, NULL, NULL);
    }
  }
  gtk_widget_destroy (dlg);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Remove texture ou heightmap
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Remove_IMG(gpointer i)
{
  if ((int)i > 0 && (int)i < 5) nbTexChargees--;
  clean_IMG((int)i);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Clean une IMG et contrôles associés
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void clean_IMG(gint i)
{
   gtk_widget_destroy(IMGs[i]->frame);
   g_free(IMGs[i]);
          IMGs[i] = NULL;

   // si l'image est chargée au central, affiche la suivante
   if (i==currentID)
   {
     gint c;
     for (c=0; c<6; c++)
     {
       if (IMGs[c])//->pixbuf)
       {
         gtk_image_set_from_pixbuf (GTK_IMAGE(mainIMG), IMGs[c]->pixbuf);
         currentID = c;
         break;
       }
     }
   }
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Clic sur une miniature : charge l'image centrale
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Miniature_Click(gpointer i)
{
  gtk_image_set_from_pixbuf (GTK_IMAGE(mainIMG), IMGs[(int)i]->pixbuf);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Charge une texture et contrôles à la table des textures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Add_item(GtkWidget *table, gchar *path, gint slot)
{
  gchar *buf;

  if (!IMGs[slot])
       IMGs[slot] = (IMG*) g_malloc(sizeof(IMG));

  IMGs[slot]->chemincomplet = g_strdup(path);

  // Frame ---------------------------------------------------------------------
     IMGs[slot]->frame = gtk_frame_new (NULL);
     gtk_table_attach_defaults (GTK_TABLE (table), IMGs[slot]->frame, 0,1, slot, slot+1);
     gtk_widget_show (IMGs[slot]->frame);
  //crée une table dans le frame -----------------------------------------------
     IMGs[slot]->tabl = gtk_table_new (3, 3, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (IMGs[slot]->tabl), 20);
     gtk_table_set_col_spacings (GTK_TABLE (IMGs[slot]->tabl), 20);
     gtk_container_add(GTK_CONTAINER(IMGs[slot]->frame),IMGs[slot]->tabl);
     gtk_widget_show (IMGs[slot]->tabl);

  // image ---------------------------------------------------------------------
     IMGs[slot]->pixbuf = gdk_pixbuf_new_from_file (path, NULL);
     //pixbuf[n] = gdk_pixbuf_new_from_file (path, NULL);

     // Redimensionne l'image si elle fait plus de 64 de côté.
     gint scaleHeight = 1, scaleWidth = 1;
     if (gdk_pixbuf_get_width(IMGs[slot]->pixbuf) > 64 )
		scaleWidth = gdk_pixbuf_get_width(IMGs[slot]->pixbuf)/64;
     if (gdk_pixbuf_get_height(IMGs[slot]->pixbuf) > 64)
		scaleHeight = gdk_pixbuf_get_height(IMGs[slot]->pixbuf)/64;

     IMGs[slot]->minipixbuf = gdk_pixbuf_scale_simple (IMGs[slot]->pixbuf,
		gdk_pixbuf_get_width(IMGs[slot]->pixbuf) / scaleWidth,
		gdk_pixbuf_get_height(IMGs[slot]->pixbuf) / scaleHeight,
		GDK_INTERP_NEAREST);
     IMGs[slot]->img = gtk_image_new_from_pixbuf (IMGs[slot]->minipixbuf);

     // la miniature est sur un bouton. clic -> sur l'image centrale
        IMGs[slot]->btn = gtk_button_new();
        gtk_button_set_relief(GTK_BUTTON(IMGs[slot]->btn), GTK_RELIEF_NONE);
        gtk_container_add (GTK_CONTAINER (IMGs[slot]->btn), IMGs[slot]->img);
        gtk_table_attach_defaults (GTK_TABLE (IMGs[slot]->tabl), IMGs[slot]->btn, 0,1, 0, 3);
        gtk_signal_connect_object (GTK_OBJECT (IMGs[slot]->btn), "clicked", G_CALLBACK(Miniature_Click), (gpointer)slot);
     gtk_widget_show (IMGs[slot]->btn);
     gtk_widget_show (IMGs[slot]->img);

     // d'ailleurs la grosse au centre est chargée dès maintenant
     gtk_image_set_from_pixbuf (GTK_IMAGE(mainIMG), IMGs[slot]->pixbuf);

  // nom -----------------------------------------------------------------------
     buf = g_path_get_basename (path);
     IMGs[slot]->labelnom = gtk_label_new (buf);
     gtk_table_attach_defaults (GTK_TABLE (IMGs[slot]->tabl), IMGs[slot]->labelnom, 1,3, 0,1);
     gtk_widget_show (IMGs[slot]->labelnom);
     buf = g_strdup_printf("%dx%d",	gdk_pixbuf_get_width(IMGs[slot]->pixbuf),
					gdk_pixbuf_get_height(IMGs[slot]->pixbuf));
  // Label taille --------------------------------------------------------------
     IMGs[slot]->labelTaille = gtk_label_new (buf);
     gtk_table_attach_defaults (GTK_TABLE (IMGs[slot]->tabl), IMGs[slot]->labelTaille, 1,2, 1,2);
     gtk_widget_show (IMGs[slot]->labelTaille);
  // Remove Button -------------------------------------------------------------
     IMGs[slot]->btnSuppr = gtk_button_new_with_label("Remove");
     gtk_signal_connect_object (GTK_OBJECT(IMGs[slot]->btnSuppr), "clicked", G_CALLBACK(Remove_IMG),(gpointer)slot);
     gtk_table_attach_defaults (GTK_TABLE(IMGs[slot]->tabl), IMGs[slot]->btnSuppr, 2,3, 1,2);
     gtk_widget_show (IMGs[slot]->btnSuppr);

  // Label Altitude ------------------------------------------------------------
     IMGs[slot]->labelAlt = gtk_label_new ("Altitude:");
     gtk_table_attach_defaults (GTK_TABLE (IMGs[slot]->tabl), IMGs[slot]->labelAlt, 1,2, 2,3);
     gtk_widget_show (IMGs[slot]->labelAlt);
  // Et son Entry --------------------------------------------------------------
     IMGs[slot]->entry = gtk_entry_new();
     gtk_widget_set_usize(IMGs[slot]->entry, 20, 20);
     gtk_entry_set_max_length (GTK_ENTRY (IMGs[slot]->entry), 6);
     buf = g_strdup_printf("%d", slot*64);
     gtk_entry_set_text (GTK_ENTRY (IMGs[slot]->entry), buf);
     g_free(buf);
     gtk_table_attach_defaults (GTK_TABLE (IMGs[slot]->tabl), IMGs[slot]->entry, 2,3, 2,3);
     gtk_widget_show (IMGs[slot]->entry);

   if (slot >0 && slot < 5) nbTexChargees++;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Choisir un fichier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
gchar* Choisir_Fichier(gchar *msg)
{
  // Creation de la fenetre de selection de fichier
     GtkWidget *Dlg = gtk_file_chooser_dialog_new(msg, NULL,
	GTK_FILE_CHOOSER_ACTION_OPEN,
	GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);

     gtk_window_set_modal(GTK_WINDOW(Dlg), TRUE);
     GtkFileFilter *FILTER = gtk_file_filter_new();
     gtk_file_filter_add_pattern (FILTER, "*.bmp");
     gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(Dlg), FILTER);

  // Affichage fenetre et récupération du path
     gchar *path=NULL;
     if (gtk_dialog_run(GTK_DIALOG(Dlg)) == GTK_RESPONSE_OK)
           path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(Dlg));
     gtk_widget_destroy(Dlg);

  return path;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Ajouter texture
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Add_Texture(GtkWidget *table)
{
  // Trouver un slot libre, en parcourant IMGs[]
  // Une texture a le slot 1-4 : 4 maximum
  gint n = 0, c=-1;
  for (n=1; n<5; n++)
  {
    if (!IMGs[n])
    {
      c = n;
      break;
    }
  }
  if (c == -1) return; // pas de slot libre: le maximum de textures est chargé.

  currentID = c;
  g_print("slot libre: %d\n", currentID);

  gchar *path = Choisir_Fichier("Ouvrir une texture...");
  if (path)
  {
    Add_item(table, path, currentID);
    g_free(path);
  }
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Ajouter Heightmap
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Add_Heightmap(GtkWidget *table)
{
  gchar *path = Choisir_Fichier("Choose Heightmap ...");
  if (path)
  {
    // if a heightmap is already loaded, first clean it up
    if (IMGs[0]) clean_IMG(0);

    // heightmap is always on slot 0
    Add_item(table, path, 0);
    g_free(path);
  }
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Crée un Label et son Entry associé
  Caption du label, index d'entry, texte entry, maxlength entry, table, Y table.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void creer_labelEntry(	gchar *caption,
			gint i, gchar* defval, gint maxlength,
			GtkWidget *table, gint y)
{
  // Label
     GtkWidget *label = gtk_label_new (caption);
     gtk_table_attach_defaults (GTK_TABLE (table), label, 0,1, y,y+1);

  // Entry
     entries[i] = gtk_entry_new();
     gtk_widget_set_usize(entries[i], 20, 20);
     gtk_entry_set_max_length (GTK_ENTRY (entries[i]), maxlength);
     gtk_entry_set_text (GTK_ENTRY (entries[i]), defval);
     gtk_table_attach_defaults (GTK_TABLE (table), entries[i], 1,2, y,y+1);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Crée un box contenant une image et un label, à intégrer dans un bouton
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
GtkWidget *CoolButton( gchar *filename, gchar *caption)
{
    GtkWidget *box;
    GtkWidget *label;
    GtkWidget *image;

    box = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (box), 2);
    image = gtk_image_new_from_file (filename);

    label = gtk_label_new (caption);

    gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 3);
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 3);
    gtk_widget_show (image);
    gtk_widget_show (label);

    return box;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Application is being closed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void App_Close()
{
   if (CFG) g_free(CFG);
   if (nbTexChargees)
   {
    guint n;
    for (n=0; n<6; n++) g_free(IMGs[n]);
   }
   gtk_main_quit();
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Main Window Creation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Init_GUI()
{
  GtkWidget *box, *scroll, *frame, *button=NULL;
  GtkWidget *tableGauche, *tableGen, *tableDroite;

  fen = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (fen), "KatSplatter");
  gtk_window_set_default_icon_from_file ("img/kat104.png",NULL);
  gtk_widget_set_usize(fen, 800, 500);
  g_signal_connect(G_OBJECT(fen), "destroy", G_CALLBACK(App_Close), NULL);

  // Window is divided in 3 columns
  box = gtk_hbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(fen), box);
  gtk_widget_show(box);


 //*****************************************************************************
 // à gauche: colonne textures/heightmaps dans un scroll ***********************
 //*****************************************************************************
   // scroll colonne images
   scroll = gtk_scrolled_window_new (NULL, NULL);
   gtk_container_set_border_width (GTK_CONTAINER (scroll), 1);
   gtk_scrolled_window_set_policy (	GTK_SCROLLED_WINDOW (scroll),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_widget_set_size_request (scroll, 260, -1);
   gtk_box_pack_start(GTK_BOX(box), scroll, FALSE, FALSE, 0);

   // table de la colonne images
   tableGauche = gtk_table_new (6, 1, FALSE);
   gtk_table_set_row_spacings (GTK_TABLE (tableGauche), 1);
   gtk_table_set_col_spacings (GTK_TABLE (tableGauche), 1);
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scroll), tableGauche);

   // Table des Heightmaps -----------------------------------------------------
     tableHmap = gtk_table_new (2, 2, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (tableHmap), 1);
     gtk_table_set_col_spacings (GTK_TABLE (tableHmap), 1);
     gtk_table_attach_defaults  (GTK_TABLE (tableGauche), tableHmap, 0,1, 1,2);
   // bouton ajouter heightmap -------------------------------------------------
     button = gtk_button_new_with_label ("Add Heightmap");
     gtk_signal_connect_object (GTK_OBJECT (button), "clicked", G_CALLBACK(Add_Heightmap), tableHmap);
     gtk_table_attach_defaults (GTK_TABLE (tableGauche), button, 0,1, 0,1);

   // Table des texture --------------------------------------------------------
     tableTex = gtk_table_new (10, 2, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (tableTex), 1);
     gtk_table_set_col_spacings (GTK_TABLE (tableTex), 1);
     gtk_table_attach_defaults  (GTK_TABLE (tableGauche), tableTex, 0,1, 3,4);
   // bouton ajouter texture ---------------------------------------------------
     button = gtk_button_new_with_label ("Add texture");
     gtk_signal_connect_object (GTK_OBJECT (button), "clicked", G_CALLBACK(Add_Texture), tableTex);
     gtk_table_attach_defaults (GTK_TABLE (tableGauche), button, 0,1, 2,3);


 //*****************************************************************************
 // Au centre, génèse et image centrale ****************************************
 //*****************************************************************************
  // un box --------------------------------------------------------------------
   GtkWidget *cbox = gtk_vbox_new(FALSE, 5);
      gtk_container_add(GTK_CONTAINER(box), cbox);
      gtk_widget_show(cbox);
      gtk_widget_set_size_request (cbox, 512, -1);

  // contient le frame génèse --------------------------------------------------
     frame = gtk_frame_new ("");//"Gen texture");
     gtk_box_pack_start(GTK_BOX(cbox), frame, FALSE, FALSE, 0);

     // une table dans le frame ------------------------------------------------
     tableGen = gtk_table_new (2, 3, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (tableGen), 20);
     gtk_table_set_col_spacings (GTK_TABLE (tableGen), 20);
     gtk_container_add(GTK_CONTAINER(frame),tableGen);

     // dans cette table, bouton générer ---------------------------------------
        button = gtk_button_new_with_label ("Splat Textures");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked", G_CALLBACK(Gen_Texture), NULL);
        gtk_table_attach_defaults (GTK_TABLE (tableGen), button, 0,1, 0,1);

     // bouton Export_File --------------------------------------------------------
        button = gtk_button_new_with_label ("Export File");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked", G_CALLBACK(Export_File), NULL);
        gtk_table_attach_defaults (GTK_TABLE (tableGen), button, 1,2, 0,1);

     // Lable et Entry Texture Factor 1 ----------------------------------------
        creer_labelEntry("Texture Factor AX:",0,"64",3,tableGen, 1);
     // Lable et Entry Texture Factor 2 ----------------------------------------
        creer_labelEntry("Texture Factor BX:",1,"64",3,tableGen, 2);

  // et l'image centrale -------------------------------------------------------
     mainIMG = gtk_image_new();//gtk_image_new_from_file ("512/herbe.bmp");
       gtk_box_pack_start(GTK_BOX(cbox), mainIMG, FALSE, FALSE, 0);


 //*****************************************************************************
 // à droite, génèse Heightmap et Lightmap *************************************
 //*****************************************************************************
   // scroll colonne de droite
   scroll = gtk_scrolled_window_new (NULL, NULL);
   gtk_container_set_border_width (GTK_CONTAINER (scroll), 1);
   gtk_scrolled_window_set_policy (	GTK_SCROLLED_WINDOW (scroll),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_widget_set_size_request (scroll, 260, -1);
   gtk_box_pack_start(GTK_BOX(box), scroll, FALSE, FALSE, 0);

   // table GenHeightmap =======================================================
   tableDroite = gtk_table_new (8, 2, FALSE);
   gtk_table_set_row_spacings (GTK_TABLE (tableDroite), 1);
   gtk_table_set_col_spacings (GTK_TABLE (tableDroite), 1);
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scroll), tableDroite);

   // Paramètres Génération Heightmap HillTerrain ==============================
     // dans cette table, bouton générer avec image robotfrog ------------------
        button = gtk_button_new();
        box = CoolButton ("img/RobotFrog.png", "Gen HillTerrain Heightmap");
        //gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked", G_CALLBACK(Gen_HillTerrain), NULL);
        gtk_table_attach_defaults (GTK_TABLE (tableDroite), button, 0,2, 0,1);

     // Les couples labels/Entry -----------------------------------------------
        creer_labelEntry("HillMin:"   ,2,"10"   ,4,tableDroite, 2);
        creer_labelEntry("HillMax:"   ,3,"40"   ,4,tableDroite, 3);
        creer_labelEntry("NumHills:"  ,4,"1000" ,5,tableDroite, 4);
        creer_labelEntry("Flattening:",5,"1"    ,4,tableDroite, 5);
        creer_labelEntry("Island:"    ,6,"0"    ,4,tableDroite, 6);
        creer_labelEntry("Seed:"      ,7,"55423",5,tableDroite, 7);

   // Paramètres génération Lightmap ===========================================
     // bouton générer ---------------------------------------------------------
        button = gtk_button_new_with_label ("Générer Lightmap");
        //gtk_signal_connect_object (GTK_OBJECT (button), "clicked", G_CALLBACK(Choisir_Fichier), tableHmap);
        gtk_table_attach_defaults (GTK_TABLE (tableDroite), button, 0,2, 8,9);

     // Les couples labels/Entry -----------------------------------------------
        creer_labelEntry("Sun Angle:"        ,8 ,"45"  ,3,tableDroite, 10);
        creer_labelEntry("Rays Max Altitude:",9 ,"2000",5,tableDroite, 11);
        creer_labelEntry("Radius:"           ,10,"2000",6,tableDroite, 12);

     /*
     // Table du Lightmap -----------------------------------------------------
         tableLmap = gtk_table_new (6, 2, FALSE);
         gtk_table_set_row_spacings (GTK_TABLE (tableLmap), 1);
         gtk_table_set_col_spacings (GTK_TABLE (tableLmap), 1);
         gtk_table_attach_defaults  (GTK_TABLE (tableDroite), tableLmap, 0,1, 8,9);

     // bouton générer ---------------------------------------------------------
        button = gtk_button_new_with_label ("Générer Lightmap");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked", G_CALLBACK(Gen_Lightmap), tableLmap);
        gtk_table_attach_defaults (GTK_TABLE (tableLmap), button, 0,2, 0,1);

     // Les couples labels/Entry -----------------------------------------------
        creer_labelEntry("Angle soleil:"     ,8 ,"45"  ,3,tableLmap, 1);
        creer_labelEntry("Altitude Max:"     ,9 ,"2000",5,tableLmap, 2);
        creer_labelEntry("Radius:"           ,10,"2000",6,tableLmap, 3);
        creer_labelEntry("Hauteur Heightmap:",11,"40"  ,6,tableLmap, 4);
       //Add_item(tableLmap, "256/lightmap.bmp", 5);
    */


 // ouch! show window now
   gtk_widget_show_all(fen);

}


