/* Splatter.cpp by Bertrand-Xavier M. March 2007 */

#include "splatter.h"

class heightmap
{
public:
	float *hmap;

	heightmap::heightmap(char *filename)
	{
	  bitmap map;
	  if (map.load (filename))
	  {
	    width  = map.getwidth();
	    height = map.getheight();

	    hmap = new float[width*height];
	    if (hmap) 
	    {
	      for (int z=0; z<height; z++) 
	      {
	        for (int x=0; x<width; x++)
	        {
	          hmap[x+(z*width)]=map.data[(x+(z*width))*3 + 1];
	        }
	      }
	    }
	  }
	}

	int heightmap::getwidth(void)
	{
		if (hmap) return width;
		else return -1;
	}

	int heightmap::getheight(void)
	{
		if (hmap) return height;
		else return -1;
	}

	heightmap::~heightmap(void)
	{
		if (hmap) delete hmap;
		hmap = 0;
	}

private:
	int width, height;
};




splatter::splatter(char *alpha, char **img, unsigned char *alt,
			unsigned char AX, unsigned char BX)
{
  tAX = AX;
  tBX = BX;

  bitmap bmp[nb], final;
  heightmap blender(alpha);

  for (int i=0; i<nb; i++) 
  {
    if (!bmp[i].load(img[i]))
    {
       printf ("couldn't load %s...\n", img[i]);
       return;
    }
    //Check that all images are the same size
    if (bmp[i].getheight() != blender.getheight())
    {
       printf ("  Images aren't of the same size, aborting.\n");
       return;
    }
  }

  //Create new bitmap
  final.create (blender.getwidth(), blender.getheight());

  // height at a given point
  float height;
  // color at that point
  unsigned char r[nb], g[nb], b[nb];
  // texture factor of each image at that point
  float tex_fact[nb];
  // Sum of textures after texfactor applied. 0=Red 1=Green 2=Blue
  float total[3];
  // new color depending on texture factor
  unsigned char newr, newg, newb;

  //Go through all pixels
  for (int y=0; y<blender.getheight(); y++)
  {
    for (int x=0; x<blender.getwidth(); x++)
    {
      //Get proper heightmap value at x/z
      height = limit255(blender.hmap[x + (y*blender.getwidth())]);

      tex_fact[0]=texture_factor(alt[0], height);
      tex_fact[1]=texture_factor(alt[1], height);
      tex_fact[2]=texture_factor(alt[2], height);
      tex_fact[3]=texture_factor(alt[3], height);

      //Read all texture rgb values
      for (int i=0; i<nb; i++) bmp[i].getcolor (x, y, r[i], g[i], b[i]);

      //Calculate new RGB value for the pixel to be written
      total[0] = total[1] = total[2] = 0;
      for (int i=0; i<4; i++)
      {
       total[0] += r[i] * tex_fact[i];
       total[1] += g[i] * tex_fact[i];
       total[2] += b[i] * tex_fact[i];
      }

      newr = (unsigned char)limit255(total[0]);
      newg = (unsigned char)limit255(total[1]);
      newb = (unsigned char)limit255(total[2]);

      //Set new pixel color in final
      final.setcolor(x, y, newr, newg, newb);
    }
  }

  //Save bitmap
  if (!final.save ("final.bmp"))
       printf ("couldn't save final image.\n");
  //else printf ("images blended, see final.bmp.\n");

}

float splatter::limit255 (float a)
{
   if     (a < 0.0f)   return 0.0f;
   else if(a > 255.0f) return 255.0f;
   else  return a;
}

// Check for percentage of color
float splatter::texture_factor(int h1, float h2)
{
  float t = ((float)tAX - abs(h1 - (int)h2)) / (float)tBX;

  if     (t < 0.0f) t = 0.0f;
  else if(t > 1.0f) t = 1.0f;

  return t;
}

splatter::~splatter()
{
}




















// interfaçage C
void createSplatter(char *alpha, char **img, unsigned char *alt,
			unsigned char AX, unsigned char BX)
{
  splatter* thesplat = new splatter(alpha, img, alt, AX, BX);
  if (thesplat) delete thesplat;
}



